
###################################################################################
#### Function loadwpfdata
###################################################################################

loadwpfdata <- function(filename) {

# function loadwpfdata('filename')
#
# This function permits to the load the wind power probabilistic forecasting
# data from the file 'filename'
#
# example: wpf_data <- loadwpfdata('mywpfdata.Rdat')

wpf <- read.table(filename, header = TRUE, sep = " ", quote = "\"'", dec = ".")
return(wpf)
}

###################################################################################
#### Function plotpredcdf
###################################################################################

plotpredcdf <- function(wpf,date,ihor) {

# function plotpredcdf(wpf,timestamp,ihor)
#
# This function permits to plot a single predictive cdf (and related measurements)
# for a given timestamp at which the forecast is issued, and a given forecast horizon.
# The probabilistic forecasts are defined in terms of quantiles.
#
# example: plotpredcdf(wpf_data,2006120510,24)
# 	   (ie. 5th of December 2006, 10am, 24 hour ahead predictive cdf)

wpf1 <- subset(wpf,wpf$dati==date,drop=FALSE) 
wpfc <- subset(wpf1,wpf1$hor==ihor,drop=FALSE) 

np   <- c(seq(0.,0.45,by=0.05),seq(0.55,1,by=0.05))
pfore <- c(wpfc$q0,wpfc$q5,wpfc$q10,wpfc$q15,wpfc$q20,wpfc$q25,wpfc$q30,wpfc$q35,wpfc$q40,wpfc$q45,wpfc$q55,wpfc$q60,wpfc$q65,wpfc$q70,wpfc$q75,wpfc$q80,wpfc$q85,wpfc$q90,wpfc$q95,wpfc$q100)

nq <-locateobs(pfore,np,wpfc$meas)

dev.new(width=7, height=5.5)
par(mar=c(3,3,0.5,1))  # Trim margin around plot [b,l,t,r]
par(tcl=0.35)  # Switch tick marks to insides of axes
par(mgp=c(1.5,0.2,0))  # Set margin lines; default c(3,1,0) [title,labels,line]
par(xaxs="r",yaxs="r")  # Extend axis limits by 4% ("i" does no extension)
plot(pfore,np,xlab="",ylab="",type='b',col="black",xlim=c(0,1),ylim=c(0,1),axes=FALSE)
par(new=TRUE)
plot(wpfc$meas,nq,xlab="power [% of Pn]",ylab="probability",type='o',col="red",xlim=c(0,1),ylim=c(0,1),axes=FALSE)
axis(1,seq(0,1,0.2),seq(0,100,20))
axis(2,seq(0,1,0.2),seq(0,1,0.2))
leg.txt <- c("predictive cdf", "observation")
legend(0, 1, leg.txt,pch="_o",col=c("black","red"))
}


###################################################################################
#### Function plotprobforec
###################################################################################

plotprobforec <- function(wpf,date) {

# function plotprobforec(wpf,timestamp)
#
# This function permits to plot probabilistic forecasts and related measurements 
# for a given timestamp at which the forecast is issued.
# The probabilistic forecasts are defined in terms of quantiles.
# Prediction intervals are shown in the form of river of blood fan charts
#
# example: plotprobforec(wpf_data,2006120510)
# 	   (ie. 5th of December 2006, 10am)

wpf1 <- subset(wpf,wpf$dati==date,drop=FALSE) 

dev.new(width=9, height=5.5)
par(mar=c(3,3,0.5,1))  # Trim margin around plot [b,l,t,r]
par(tcl=0.35)  # Switch tick marks to insides of axes
par(mgp=c(1.5,0.2,0))  # Set margin lines; default c(3,1,0) [title,labels,line]
par(xaxs="r",yaxs="r")  # Extend axis limits by 4% ("i" does no extension)
plot(wpf1$hor,wpf1$pred,xlab="lead time [hours]",ylab="power [%]",type='l',col="red",ylim=c(0,1),axes=FALSE)
par(new=TRUE)
conf_region(wpf1$hor,wpf1$hor, wpf1$hor, wpf1$q5, wpf1$q95, fill_col="gray90")
par(new=TRUE)
conf_region(wpf1$hor,wpf1$hor, wpf1$hor, wpf1$q10, wpf1$q90, fill_col="gray80")
par(new=TRUE)
conf_region(wpf1$hor,wpf1$hor, wpf1$hor, wpf1$q15, wpf1$q85, fill_col="gray70")
par(new=TRUE)
conf_region(wpf1$hor,wpf1$hor, wpf1$hor, wpf1$q20, wpf1$q80, fill_col="gray60")
par(new=TRUE)
conf_region(wpf1$hor,wpf1$hor, wpf1$hor, wpf1$q25, wpf1$q75, fill_col="gray50")
par(new=TRUE)
conf_region(wpf1$hor,wpf1$hor, wpf1$hor, wpf1$q30, wpf1$q70, fill_col="gray40")
par(new=TRUE)
conf_region(wpf1$hor,wpf1$hor, wpf1$hor, wpf1$q35, wpf1$q65, fill_col="gray50")
par(new=TRUE)
conf_region(wpf1$hor,wpf1$hor, wpf1$hor, wpf1$q40, wpf1$q60, fill_col="gray30")
par(new=TRUE)
conf_region(wpf1$hor,wpf1$hor, wpf1$hor, wpf1$q45, wpf1$q55, fill_col="gray20")
par(new=TRUE)
plot(wpf1$hor,wpf1$meas,xlab="",ylab="",col="green",ylim=c(0,1),axes=FALSE)
axis(1,c(1,seq(6,42,6)),c(1,seq(6,42,6)))
axis(2,seq(0,1,0.2),seq(0,100,20))
leg.txt <- c("10-90% pred. intervals", "observations")
legend(1, 1, leg.txt,pch=c(15,1),col=c("gray60","green"))
}

###################################################################################
#### Function conf_region
###################################################################################

#---Plot (shaded) confidence region
conf_region <- function(x, xlo, xup, ylo, yup, fill_col="grey", border=par("bg") , lcol=par("bg"), ... )
{
	N <- length(x)
	stopifnot(length(xlo)==N)
	stopifnot(length(xup)==N)
	stopifnot(length(ylo)==N)
	stopifnot(length(yup)==N)

        #--Get plot axis limits
	lims <- par("usr")
	if ( par("xlog") ) lims[1:2] <- 10^lims[1:2]
	if ( par("ylog") ) lims[3:4] <- 10^lims[3:4]
	xax.lo <- lims[1]; xax.up <- lims[2]
	yax.lo <- lims[3]; yax.up <- lims[4]

	#--Calculate coordinates of polygon vertices:
	X <- c(xlo[1],x,xup[N])
	Y1 <- c(ylo[1],ylo,ylo[N])
	Y2 <- c(yup[1],yup,yup[N])

	# Plot shaded region with supplied border colour:
	polygon(c(X,rev(X)),c(Y1,rev(Y2)),col=fill_col,border=border,ylim=c(0,1),...)

	#--Plot top & bottom bounding edges with supplied colour:
	lines(X,Y1,col=lcol,ylim=c(0,1))
	lines(X,Y2,col=lcol,ylim=c(0,1))
}


###################################################################################
#### Function reldiag
###################################################################################

reldiag <- function(wpf){

# function reldiag(wpf)
#
# This function permits to visualize the reliability diagram related to 
# the calibration assessment of the probabilistic forecasts in 'wpf'. 
# These reliability diagrams take the form of Quantile-Quantile plots.
#
# example: reldiag(wpf_data)

erel <- releval(wpf)
nrel = c(seq(0.05,0.45,by=0.05),seq(0.55,0.95,by=0.05))

dev.new(width=6, height=6)
par(mar=c(3,3,0.5,1))  # Trim margin around plot [b,l,t,r]
par(tcl=0.35)  # Switch tick marks to insides of axes
par(mgp=c(1.5,0.2,0))  # Set margin lines; default c(3,1,0) [title,labels,line]
par(xaxs="r",yaxs="r")  # Extend axis limits by 4% ("i" does no extension)
plot(nrel,nrel,xlab="nominal proportions",ylab="observed proportions",type='l',col="black",xlim=c(0,1),ylim=c(0,1),axes=FALSE)
par(new=TRUE)
plot(nrel,erel,xlab="",ylab="",type='b',col="blue",xlim=c(0,1),ylim=c(0,1),axes=FALSE)
axis(1,seq(0,1,0.2),seq(0,1,0.2))
axis(2,seq(0,1,0.2),seq(0,1,0.2))

}


###################################################################################
#### Function reldiagcomp
###################################################################################

reldiagcomp <- function(wpf1,wpf2,wpf3){

# function reldiagcomp(wpf1,wpf2,wpf3)
#
# This function permits to visualize comparative reliability diagrams related 
# to the calibration assessment of the probabilistic forecasts in 'wpf1', 
# 'wpf2', and 'wpf3'. These reliability diagrams take the form of 
# Quantile-Quantile plots.
#
# example: reldiagcomp(wpf_data1,wpf_data2,wpf_data3)

nrel = c(seq(0.05,0.45,by=0.05),seq(0.55,0.95,by=0.05))
erel1 <- releval(wpf1)
erel2 <- releval(wpf2)
erel3 <- releval(wpf3)

dev.new(width=6, height=6)
par(mar=c(3,3,0.5,1))  # Trim margin around plot [b,l,t,r]
par(tcl=0.35)  # Switch tick marks to insides of axes
par(mgp=c(1.5,0.2,0))  # Set margin lines; default c(3,1,0) [title,labels,line]
par(xaxs="r",yaxs="r")  # Extend axis limits by 4% ("i" does no extension)
plot(nrel,nrel,xlab="nominal proportions",ylab="observed proportions",type='l',col="gray50",xlim=c(0,1),ylim=c(0,1),axes=FALSE)
par(new=TRUE)
plot(nrel,erel1,xlab="",ylab="",type='b',col="blue",xlim=c(0,1),ylim=c(0,1),axes=FALSE)
par(new=TRUE)
plot(nrel,erel2,xlab="",ylab="",type='b',col="red",xlim=c(0,1),ylim=c(0,1),axes=FALSE)
par(new=TRUE)
plot(nrel,erel3,xlab="",ylab="",type='b',col="black",xlim=c(0,1),ylim=c(0,1),axes=FALSE)
leg.txt <- c("forecast set 1","forecast set 2",'forecast set 3')
legend(0, 1, leg.txt,pch="___",col=c("blue","red","black"))
axis(1,seq(0,1,0.2),seq(0,1,0.2))
axis(2,seq(0,1,0.2),seq(0,1,0.2))

}

###################################################################################
#### Function reldiaghor
###################################################################################

reldiaghor <- function(wpf,ihor){

# function reldiaghor(wpf,ihor)
#
# This function permits to visualize the reliability diagram related to 
# the calibration assessment of the probabilistic forecasts in 'wpf', for 
# a specific forecast horizon 'ihor'. 
# These reliability diagrams take the form of Quantile-Quantile plots.
#
# example: reldiaghor(wpf_data,24) 
# 	   (ie. for 24-hour ahead probabilistic forecasts)

wpfc <- subset(wpf,wpf$hor==ihor,drop=FALSE)
erel <- releval(wpfc)

nrel <- c(seq(0.05,0.45,by=0.05),seq(0.55,0.95,by=0.05))

dev.new(width=6, height=6)
par(mar=c(3,3,0.5,1))  # Trim margin around plot [b,l,t,r]
par(tcl=0.35)  # Switch tick marks to insides of axes
par(mgp=c(1.5,0.2,0))  # Set margin lines; default c(3,1,0) [title,labels,line]
par(xaxs="r",yaxs="r")  # Extend axis limits by 4% ("i" does no extension)
plot(nrel,nrel,xlab="nominal proportions",ylab="observed proportions",type='l',col="black",xlim=c(0,1),ylim=c(0,1),axes=FALSE)
par(new=TRUE)
plot(nrel,erel,xlab="",ylab="",type='b',col="blue",xlim=c(0,1),ylim=c(0,1),axes=FALSE)
axis(1,seq(0,1,0.2),seq(0,1,0.2))
axis(2,seq(0,1,0.2),seq(0,1,0.2))

}


###################################################################################
#### Function reldiaghorcomp
###################################################################################

reldiaghorcomp <- function(wpf1,wpf2,wpf3,ihor){

# function reldiaghorcomp(wpf1,wpf2,wpf3,ihor)
#
# This function permits to visualize comparative reliability diagrams related 
# to the calibration assessment of the probabilistic forecasts in 'wpf1', 
# 'wpf2', and 'wpf3', and this for a specific forecast horizon 'ihor'. 
# These reliability diagrams take the form of Quantile-Quantile plots.
#
# example: reldiaghorcomp(wpf_data1,wpf_data2,wpf_data3,24) 
# 	   (ie. for 24-hour ahead probabilistic forecasts)

wpfc1 <- subset(wpf1,wpf1$hor==ihor,drop=FALSE)
wpfc2 <- subset(wpf2,wpf2$hor==ihor,drop=FALSE)
wpfc3 <- subset(wpf3,wpf3$hor==ihor,drop=FALSE)
nrel = c(seq(0.05,0.45,by=0.05),seq(0.55,0.95,by=0.05))
erel1 <- releval(wpfc1)
erel2 <- releval(wpfc2)
erel3 <- releval(wpfc3)

dev.new(width=6, height=6)
par(mar=c(3,3,0.5,1))  # Trim margin around plot [b,l,t,r]
par(tcl=0.35)  # Switch tick marks to insides of axes
par(mgp=c(1.5,0.2,0))  # Set margin lines; default c(3,1,0) [title,labels,line]
par(xaxs="r",yaxs="r")  # Extend axis limits by 4% ("i" does no extension)
plot(nrel,nrel,xlab="nominal proportions",ylab="observed proportions",type='l',col="gray50",xlim=c(0,1),ylim=c(0,1),axes=FALSE)
par(new=TRUE)
plot(nrel,erel1,xlab="",ylab="",type='b',col="blue",xlim=c(0,1),ylim=c(0,1),axes=FALSE)
par(new=TRUE)
plot(nrel,erel2,xlab="",ylab="",type='b',col="red",xlim=c(0,1),ylim=c(0,1),axes=FALSE)
par(new=TRUE)
plot(nrel,erel3,xlab="",ylab="",type='b',col="black",xlim=c(0,1),ylim=c(0,1),axes=FALSE)
leg.txt <- c("forecast set 1","forecast set 2",'forecast set 3')
legend(0, 1, leg.txt,pch="___",col=c("blue","red","black"))
axis(1,seq(0,1,0.2),seq(0,1,0.2))
axis(2,seq(0,1,0.2),seq(0,1,0.2))


}


###################################################################################
#### Function releval
###################################################################################

releval <- function(wpf){

wpfc <- subset(wpf,wpf$pred>-98,drop=FALSE)

n  <- length(wpfc$meas)
nt <- c(sum(wpfc$meas<wpfc$q5))
nt <- c(nt,sum(wpfc$meas<wpfc$q10))
nt <- c(nt,sum(wpfc$meas<wpfc$q15))
nt <- c(nt,sum(wpfc$meas<wpfc$q20))
nt <- c(nt,sum(wpfc$meas<wpfc$q25))
nt <- c(nt,sum(wpfc$meas<wpfc$q30))
nt <- c(nt,sum(wpfc$meas<wpfc$q35))
nt <- c(nt,sum(wpfc$meas<wpfc$q40))
nt <- c(nt,sum(wpfc$meas<wpfc$q45))
nt <- c(nt,sum(wpfc$meas<wpfc$q55))
nt <- c(nt,sum(wpfc$meas<wpfc$q60))
nt <- c(nt,sum(wpfc$meas<wpfc$q65))
nt <- c(nt,sum(wpfc$meas<wpfc$q70))
nt <- c(nt,sum(wpfc$meas<wpfc$q75))
nt <- c(nt,sum(wpfc$meas<wpfc$q80))
nt <- c(nt,sum(wpfc$meas<wpfc$q85))
nt <- c(nt,sum(wpfc$meas<wpfc$q90))
nt <- c(nt,sum(wpfc$meas<wpfc$q95))
erel <- nt/n
return(erel)

}

###################################################################################
#### Function crpscomp
###################################################################################

crpscomp <- function(wpf1,wpf2,wpf3){

# function crpscomp(wpf1,wpf2,wpf3)
#
# This function permits to compare the CRPS scores (as a function of the forecast 
# horizon), for the comparative overvall skill assessment of the probabilistic 
# forecasts in 'wpf1', 'wpf2' and 'wpf3'. 
#
# example: crpscomp(wpf_data1,wpf_data2,wpf_data3)

nhor <- max(wpf1$hor)

crpsihor1 <- rep(0,nhor)
crpsihor2 <- rep(0,nhor)
crpsihor3 <- rep(0,nhor)

for (i in 1:nhor) {
	wpfihor1 <- subset(wpf1,wpf1$hor==i,drop=FALSE)
	crpsihor1[i] <- 100*crpsf(wpfihor1)
	wpfihor2 <- subset(wpf2,wpf2$hor==i,drop=FALSE)
	crpsihor2[i] <- 100*crpsf(wpfihor2)
	wpfihor3 <- subset(wpf3,wpf3$hor==i,drop=FALSE)
	crpsihor3[i] <- 100*crpsf(wpfihor3)
}

dev.new(width=9, height=5.5)
par(mar=c(3,3,0.5,1))  # Trim margin around plot [b,l,t,r]
par(tcl=0.35)  # Switch tick marks to insides of axes
par(mgp=c(1.5,0.2,0))  # Set margin lines; default c(3,1,0) [title,labels,line]
par(xaxs="r",yaxs="r")  # Extend axis limits by 4% ("i" does no extension)
plot(crpsihor1,xlab="lead time [hours]",ylab="CRPS [% of Pn]",type='l',col="blue",ylim=c(0,15),axes=FALSE)
par(new=TRUE)
plot(crpsihor2,xlab="",ylab="",type='l',col="red",ylim=c(0,15),axes=FALSE)
par(new=TRUE)
plot(crpsihor3,xlab="",ylab="",type='l',col="black",ylim=c(0,15),axes=FALSE)
leg.txt <- c("forecast set 1","forecast set 2",'forecast set 3')
legend(1, 10, leg.txt,pch="___",col=c("blue","red","black"))
axis(1,c(1,seq(6,42,6)),c(1,seq(6,42,6)))
axis(2,seq(0,15,3),seq(0,15,3))
}


###################################################################################
#### Function crps
###################################################################################

crps <- function(wpf){

# function crps(wpf)
#
# This function permits to visualize the CRPS as a function of the forecast horizon, 
# for the overvall skill assessment of the probabilistic forecasts in 'wpf'. 
#
# example: crps(wpf_data)

nhor <- max(wpf$hor)

crpsihor <- rep(0,nhor)

for (i in 1:nhor) {
	wpfihor <- subset(wpf,wpf$hor==i,drop=FALSE)
	crpsihor[i] <- 100*crpsf(wpfihor)
}

dev.new(width=9, height=5.5)
par(mar=c(3,3,0.5,1))  # Trim margin around plot [b,l,t,r]
par(tcl=0.35)  # Switch tick marks to insides of axes
par(mgp=c(1.5,0.2,0))  # Set margin lines; default c(3,1,0) [title,labels,line]
par(xaxs="r",yaxs="r")  # Extend axis limits by 4% ("i" does no extension)
plot(crpsihor,xlab="lead time [hours]",ylab="CRPS [% of Pn]",type='l',col="blue",ylim=c(0,15),axes=FALSE)
axis(1,c(1,seq(6,42,6)),c(1,seq(6,42,6)))
axis(2,seq(0,15,3),seq(0,15,3))

}

###################################################################################
#### Function crpsf
###################################################################################

crpsf <- function(wpf){

wpfc <- subset(wpf,wpf$pred>-98,drop=FALSE)

nl = length(wpfc$meas)
crpsi <- rep(0,nl)
for (i in 1:nl) {
	pfore <-
	c(0,wpfc$q5[i], wpfc$q10[i], wpfc$q15[i], wpfc$q20[i],
	wpfc$q25[i], wpfc$q30[i], wpfc$q35[i], wpfc$q40[i],
	wpfc$q45[i], wpfc$q55[i], wpfc$q60[i], wpfc$q65[i],
	wpfc$q70[i], wpfc$q75[i], wpfc$q80[i], wpfc$q85[i], 
	wpfc$q90[i], wpfc$q95[i],1)
	obs <- wpfc$meas[i]
	nrel = c(seq(0.,0.45,by=0.05),seq(0.55,1,by=0.05))
	
	crpsi[i] <- crpsint(pfore,nrel,obs)
	
}

crpsm = sum(crpsi)/nl
return(crpsm)
}

###################################################################################
#### Function crpsint
###################################################################################

crpsint <- function(pfore,np,obs){

# Locate observation
nlo <- length(pfore)
ilo <- sum(pfore<=obs)

if (ilo==length(pfore))
	obsl <- 1
else
	dup <- pfore[ilo+1]-obs
	ddo <- obs-pfore[ilo]
	obsl <- (dup*np[ilo]+ddo*np[ilo+1])/(dup+ddo)
end

pforeo  <- sort(c(pfore,obs))
plength <- pforeo[2:(nlo+1)]-pforeo[1:nlo]

# Heavyside function
one <- rep(1,nlo-ilo+1)

# Numerical integration
vdo <- c(np[1:ilo],obsl)^2
vdu <- (one-c(obsl,np[(ilo+1):nlo]))^2
po  <- plength[1:ilo]
pu  <- plength[(ilo+1):length(plength)]

co =  sum((vdo[1:(length(vdo)-1)]+vdo[2:length(vdo)])*po/2)
cu =  sum((vdu[1:(length(vdu)-1)]+vdu[2:length(vdu)])*pu/2)

crpsi <- co+cu
return(crpsi)
}

###################################################################################
#### Function sharpness
###################################################################################

sharpness <- function(wpf){

# function sharpness(wpf)
#
# This function permits to calculate sharpness as a function of the forecast horizon, 
# sharpness being defined as the width of prediction intervals. 
#
# example: crps(wpf_data)

wpfc <- subset(wpf,wpf$pred>-98,drop=FALSE)

sharp <- rep(0,9)
sharp[1] = mean(wpfc$q55-wpfc$q45)
sharp[2] = mean(wpfc$q60-wpfc$q45)
sharp[3] = mean(wpfc$q65-wpfc$q45)
sharp[4] = mean(wpfc$q70-wpfc$q45)
sharp[5] = mean(wpfc$q75-wpfc$q45)
sharp[6] = mean(wpfc$q80-wpfc$q45)
sharp[7] = mean(wpfc$q85-wpfc$q45)
sharp[8] = mean(wpfc$q90-wpfc$q45)
sharp[9] = mean(wpfc$q95-wpfc$q45)

return(sharp)
}

###################################################################################
#### Function sharpnesscomp
###################################################################################

sharpnesscomp <- function(wpf1,wpf2,wpf3){

# function sharpnesscomp(wpf1,wpf2,wpf3)
#
# This function permits to visually compare the sharpness of 3 different forecast systems, 
# sharpness being defined as the width of prediction intervals. 
#
# example: sharpnesscomp(wpf_data1,wpf_data2,wpf_data3)

sharp1 <-sharpness(wpf1)
sharp2 <-sharpness(wpf2)
sharp3 <-sharpness(wpf3)

dev.new(width=9, height=5.5)
par(mar=c(3,3,0.5,1))  # Trim margin around plot [b,l,t,r]
par(tcl=0.35)  # Switch tick marks to insides of axes
par(mgp=c(1.5,0.2,0))  # Set margin lines; default c(3,1,0) [title,labels,line]
par(xaxs="r",yaxs="r")  # Extend axis limits by 4% ("i" does no extension)
plot(c(seq(10,90,by=10)),sharp1,xlab="nominal coverage [%]",ylab="prediction interval width [% of Pn]",type='l',col="blue",ylim=c(0,0.4),axes=FALSE)
par(new=TRUE)
plot(c(seq(10,90,by=10)),sharp2,xlab="",ylab="",type='l',col="red",ylim=c(0,0.4),axes=FALSE)
par(new=TRUE)
plot(c(seq(10,90,by=10)),sharp3,xlab="",ylab="",,type='l',col="black",ylim=c(0,0.4),axes=FALSE)
leg.txt <- c("forecast set 1","forecast set 2",'forecast set 3')
legend(10, 0.4, leg.txt,pch="___",col=c("blue","red","black"))
axis(1,seq(10,90,10),seq(10,90,10))
axis(2,seq(0,0.4,0.1),seq(0,40,10))

}

###################################################################################
#### Function locateobs
###################################################################################

locateobs <- function(pfore,np,obs){

io <- sum(pfore<=obs)
if (io==length(pfore))
	obsl <- 1
else
	dup <- pfore[io+1]-obs
	ddo <- obs-pfore[io]
	obsl <- (dup*np[io]+ddo*np[io+1])/(dup+ddo)
end
return(obsl)

}

###################################################################################
#### Function tradeprobforecast
###################################################################################

tradeprobforecast <- function(wpf,nomp,qbid,ps,cu,cd){

# function tradeprobforecast(wpf,nomp,qbid,ps,cu,cd)
#
# This function permits to trade a specific quantile of probabilistic
# forecasts (with nominal proportion 'qbid') in the electricity market,
# for a wind farm of nominal power 'nomp' MW. The spot price is given
# by 'ps', while the penalties for actually producing more or less
# than your bid are given by 'cu' and 'cd', respectively. If
# 'qbid=-1', the optimal quantile is automatically defined...
#
# example: tradeprobforecast(wpf_set,10,0.5,40,10,30)

wpfcc <- subset(wpf,wpf$pred>-98,drop=FALSE)
wpfcd <- subset(wpfcc,wpfcc$hor>12,drop=FALSE)
wpfcs  <- subset(wpfcd,wpfcd$hor<38,drop=FALSE)
wpfc <- subset(wpfcs,(wpfcs$dati-floor(wpfcs$dati/100)*100)==10,drop=FALSE)

if ((qbid<0)||(qbid>1)) {
   aq  <- cu/(cu+cd)
}
else {
   aq <- qbid
}

print(c("Proportion of traded quantile: ",aq))
qopts <- extractquantile(wpfc,aq)

bulkvalue <- sum(wpfc$meas*ps)
print(c("Revenue if perfect foresight: ",nomp*bulkvalue))

err <- wpfc$meas-qopts
pu  <- sum(err*(err>0)*cu)
pd  <- sum(err*(err<0)*(-cd))

#print(c("Volume of over-production: ", nomp*sum(err*(err>0))))
#print(c("Volume of under-production: ", -nomp*sum(err*(err<0))))
#print(c("Penalties for over-production: ",nomp*pu))
#print(c("Penalties for under-production: ",nomp*pd))
print(c("Your revenue: ",nomp*(bulkvalue-pu-pd)))
print(c("Your performance ratio: ",(bulkvalue-pu-pd)/bulkvalue))


}

###################################################################################
#### Function extractquantile
###################################################################################

extractquantile <- function(wpf,qn){

if((qn>=0)&&(qn<0.05)){
	srd <- wpf$q0
	sru <- wpf$q5
	do = 0
	du = 0.05
}
if((qn>=0.05)&&(qn<0.1)){
	srd <- wpf$q5
	sru <- wpf$q10
	do = 0.05
	du = 0.1
}
if((qn>=0.1)&&(qn<0.15)){
	srd <- wpf$q10
	sru <- wpf$q15
	do = 0.1
	du = 0.15
}
if((qn>=0.15)&&(qn<0.20)){
	srd <- wpf$q15
	sru <- wpf$q20
	do = 0.15
	du = 0.2
}
if((qn>=0.2)&&(qn<0.25)){
	srd <- wpf$q20
	sru <- wpf$q25
	do = 0.2
	du = 0.25
}
if((qn>=0.25)&&(qn<0.30)){
	srd <- wpf$q25
	sru <- wpf$q30
	do = 0.25
	du = 0.3
}
if((qn>=0.3)&&(qn<0.35)){
	srd <- wpf$q30
	sru <- wpf$q35
	do = 0.3
	du = 0.35
}
if((qn>=0.35)&&(qn<0.40)){
	srd <- wpf$q35
	sru <- wpf$q40
	do = 0.35
	du = 0.4
}
if((qn>=0.4)&&(qn<0.45)){
	srd <- wpf$q40
	sru <- wpf$q45
	do = 0.4
	du = 0.45
}
if((qn>=0.45)&&(qn<0.55)){
	srd <- wpf$q45
	sru <- wpf$q55
	do = 0.45
	du = 0.55
}
if((qn>=0.55)&&(qn<0.60)){
	srd <- wpf$q55
	sru <- wpf$q60
	do = 0.55
	du = 0.6
}
if((qn>=0.6)&&(qn<0.65)){
	srd <- wpf$q60
	sru <- wpf$q65
	do = 0.6
	du = 0.65
}
if((qn>=0.65)&&(qn<0.70)){
	srd <- wpf$q65
	sru <- wpf$q70
	do = 0.65
	du = 0.7
}
if((qn>=0.7)&&(qn<0.75)){
	srd <- wpf$q70
	sru <- wpf$q75
	do = 0.7
	du = 0.75
}
if((qn>=0.75)&&(qn<0.80)){
	srd <- wpf$q75
	sru <- wpf$q80
	do = 0.75
	du = 0.8
}
if((qn>=0.8)&&(qn<0.85)){
	srd <- wpf$q80
	sru <- wpf$q85
	do = 0.8
	du = 0.85
}
if((qn>=0.85)&&(qn<0.90)){
	srd <- wpf$q85
	sru <- wpf$q90
	do = 0.85
	du = 0.9
}
if((qn>=0.90)&&(qn<0.95)){
	srd <- wpf$q90
	sru <- wpf$q95
	do = 0.9
	du = 0.95
}
if((qn>=0.95)&&(qn<0.100)){
	srd <- wpf$q95
	sru <- wpf$q100
	do = 0.9
	du = 1
}

qseries <- (srd*(du-qn) + sru*(qn-do))/(du-do)
return(qseries)
}


###################################################################################
#### Function gr_dev
###################################################################################

grdev <- function(...) {	
get(getOption("device"))(...)
}	
